/* -----------     I2C_CTRL_TXLCD.AVC    ------------------

     Demoprogramm zum Ansteuern eines Textdisplays ueber
     I2C.

     MCU:          ATmega8   / ATmega48  / ATmega88 /
                   ATmega168 / ATmega328

     I2C-Adresse:      0x34  (siehe Defines)

     hier:             ATmega168
     Quarz:            16MHz
     
     Fuses             Lfuse 0xEF
                       Hfuse 0xDD
                       EFuse 0x01

     Compiler:         AVR-GCC 4.7.2
     IDE:              Context
     
     uebersetzen mit:  GCCCOMPILE.BAT
 

     16.06.2013        R. Seelig

   ------------------------------------------------------- */


#ifndef F_CPU
#define F_CPU 16000000
#endif

// ------------------------------------------------------------------------------------------

#define i2c_addr           0x34

// ------------------------------------------------------------------------------------------

#include <util/delay.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdlib.h>

#include "i2c_master_mega.inc"


#define chbit(wert,nr)      ((wert) & (1<<nr))               // testet auf einer Bitposition <nr> im Port <wert> auf 1 oder 0
#define prints(tx)          (putromstring(PSTR(tx)))         // Anzeige String aus Flashrom: prints("Text");

char wherex, wherey;
uint16_t count;

void delay(char wert)
{
   char i;
   for (i=0;i< wert;i++)
   {
     _delay_ms(100);
   }
}


void i2c_lcd(char cmd1, char cmd2)
{

  i2c_start(i2c_addr);
  i2c_write(0x00);
  i2c_write(cmd1);
  i2c_write(cmd2);
  i2c_write(1);
  i2c_stop();
  i2c_start(i2c_addr);
  i2c_write(0x00);
  i2c_write(cmd1);
  i2c_write(cmd2);
  i2c_write(1);
  i2c_stop();
  _delay_ms(3);
}

void tx_putchar(char ch)
{
  i2c_lcd(10,ch);
  wherex++;
}

void clrscr()
{
  i2c_lcd(3,0);
  wherex= 0; wherey= 0;
  _delay_ms(50);
}

void gotoxy(char x, char y)
{
  if (y>2) return;
  i2c_lcd(y,x);
  _delay_ms(10);
}

void putramstring(char *c)                              // Uebergabe eines Zeigers (auf einen String)
{
  while (*c)
  {
    tx_putchar(*c++);
  }
}

void putromstring(const unsigned char *dataPtr)
{
  unsigned char c;

  for (c=pgm_read_byte(dataPtr); c; ++dataPtr, c=pgm_read_byte(dataPtr)) tx_putchar(c);
}

void printint(int wert)
{
  char str16[16];

  itoa(wert,str16,10);
  putramstring(str16);
}


int main()
{

  int i=0;

  i2c_master_init();
  wherex= 0; wherey= 0;

  clrscr();
  gotoxy(3,1);
  prints("I2C:");
  gotoxy(1,2);
  prints("LC-Disp.");
  delay(15);
  clrscr();
  prints(" ohne");
  gotoxy(2,2); prints("Quarz");
  delay(15);

  count= 0;
  clrscr();
  prints("Zaehler:");
  while(1)
  {

    gotoxy(3,2);
    if (!count)
    {
      prints("     ");
      gotoxy(3,2);
    }
    printint(count);
    count++;
    delay(1);
  }
}
