/* ----------------  I2C_DISPLAY_USI.C    ----------------

     Textdisplay  wird ueber ATtiny2313 als I2C Modul
     (Slave) betrieben.


     Slaveadresse der Schaltung: !!!! 0x34 !!!!

     Display:          Textdisplay mit HD44780 Controller
                       Betriebsmodus 4-Bit

                       Pollin Display C0802-04
                       (billiger gehts nicht)

     MCU:              ATtiny2313

     Quarz:            keiner / int. Takt 8 MHz

     Fuses             Lfuse 0xE4
                       Hfuse 0xDF
                       EFuse 0xFF

     Compiler:         AVR-GCC  4.7.2
     IDE:              Context
     
     uebersetzen mit:  GCCCOMPILE.BAT


     16.06.2013        R. Seelig

   ------------------------------------------------------- */

/*
      Anschluss Pollin-Display C0802-04 an den Controller
      ---------------------------------------------------
         o +5V
         |                            Display                 Controller
         _                        Funktion   PIN            PIN    Funktion
        | |
        |_|                          GND      1 ------------
         |                          +5V       2 ------------
         o----o Kontrast   ---    Kontrast    3 ------------
        _|_                           RS      4 ------------  9      PD5
        \ /                          GND      5 ------------
        -'-                    (Takt) E       6 ------------  8      PD4
         |                           D4       7 ------------  2      PD0
        --- GND                      D5       8 ------------  3      PD1
                                     D6       9 ------------  6      PD2
                                     D7      10 ------------  7      PD3
                                                                                 I2C

                                                              19     ----------- SCL
                                                              17     ----------- SDA
*/
// ------------------------------------------------------------------------------------------------------------------


#ifndef F_CPU
#define F_CPU 8000000
#endif


#include <util/delay.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdlib.h>


#include "tx_lcd.inc"
#include "i2c_slave_usi.inc"

#define chbit(wert,nr)      ((wert) & (1<<nr))              // testet an der Bitposition <nr> im Port <wert>
                                                            // auf 1 oder 0
#define clrcmd()            (rxbuffer[0] = 0)

// -------------------------- I2C - Adresse des "Displaymoduls" -----------------------------------------------------

                                                            // Adresse auf die der ATtiny auf dem I2C-Bus "hoert"
#define slave_addr          0x34                            // muss eine "gerade" Adresse sein, da das niederwertigste
                                                            // Bit bestimmt, ob der Bus gelesen oder geschrieben wird

// ------------------------------------------------------------------------------------------------------------------

char b1,b2,b3;

int main()
{

  lcd_init();

  usiTwiSlaveInit(slave_addr);              // TWI/I2C Slave-Adresse
  sei();

  for (int i = 0; i < 17; i++)              // Puffer auf 0 setzen
    txbuffer[i] = 0;

  for (int i = 0; i < 4; i++)
    rxbuffer[i] = 0;

  b3= 0;

  lcd_putchar(( (slave_addr>>4) & 0xf)+48);
  lcd_putchar(( slave_addr & 0xf)+48);
  lcd_putchar('h');
  while(1)
  {
    rxbuffer[2]= 0;
    b3= 0;
    while(b3==0)
    {
      b1= rxbuffer[0];
      b2= rxbuffer[1];
      b3= rxbuffer[2];
      _delay_ms(2);
    }


      switch (b1)
      // Hier koennen weitere Kommandos eingefuegt werden, die dann der I2C-Master senden kann
      // bspw. koennte das Display um die "Verschiebemoeglichkeiten" erweitert werden
      {
        case  1:  { gotoxy(b2,1);
                    break; }
        case  2:  { gotoxy(b2,2);
                    break; }
        case  3:  { lcd_init();
                    break; }
        case 10:  { lcd_putchar(b2);
                    break; }
        default:  {
                    break; }
      }

  }
}
